package com.hzya.frame.u8.saleout.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.crm.util.CrmUtil;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.u8.saleinv.dao.ISaleInvDao;
import com.hzya.frame.u8.saleinv.entity.SaleInvEntity;
import com.hzya.frame.u8.saleout.dao.ISaleOutDao;
import com.hzya.frame.u8.saleout.dao.ISaleOutDetailDao;
import com.hzya.frame.u8.saleout.dao.impl.SaleOutDetailDaoImpl;
import com.hzya.frame.u8.saleout.entity.SaleOutDetailEntity;
import com.hzya.frame.u8.saleout.entity.SaleOutEntity;
import com.hzya.frame.u8.saleout.service.ISaleOutService;
import com.hzya.frame.u8.util.BaseU8Result;
import com.hzya.frame.u8.util.U8ResultEntity;
import com.hzya.frame.u8.util.U8Util;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @content 销售出库单
 * @className: Administrator
 * @author laborer
 * @date 2024-10-18 9:27
 *
 */
@Service(value = "SaleOutServiceImpl")
public class SaleOutServiceImpl extends BaseService<SaleOutEntity,String> implements ISaleOutService {
    @Autowired
    private ISaleInvDao saleInvDao;
    Logger logger = LogManager.getLogger(getClass());
    @Autowired
    private ISaleOutDao saleOutDao;
    @Autowired
    private ISaleOutDetailDao saleOutDetailDao;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;

    /**
     *
     * @content 获取U8销售出库单到CRM
     * @className: Administrator
     * @author laborer
     * @date 2024-10-18 10:15
     *
     */

    @Override
    public void getU8SaleOutToCrm(JSONObject requestJson) {
        //获取重推信息id，并记录日志
        JSONObject requestJsonObj = JSON.parseObject(requestJson.toString(),JSONObject.class);
        String interId = requestJsonObj.getString("integration_task_living_details_id");
        logger.info("U8销售出库单重推解析后："+requestJsonObj.toString());
        logger.info("U8销售出库单推送CRM重新执行的任务主键："+interId);

        //更换数据源
        JsonResultEntity resultEntity = new JsonResultEntity();
        SaleOutEntity entity = new SaleOutEntity();
        requestJson.put("db_code","YONYOUU8");
        entity.setDataSourceCode(requestJson.getString("db_code"));

        //根据重推信息id，查询该条重推记录信息
        IntegrationTaskLivingDetailsEntity oldMsg = new IntegrationTaskLivingDetailsEntity();
        if(StrUtil.isNotEmpty(interId)){
            oldMsg = taskLivingDetailsService.get(interId);
        }
        String rootAppPk = oldMsg.getRootAppPk();
        entity.setIds(rootAppPk);
        StrUtil.isEmpty(rootAppPk);//entity.setQueryState("查询");
        //如果是重推，是否通过entity传入的id，只能查到一条数据。
        //entity.setId("-8845102890648320822");
        List<SaleOutEntity> soleList = null;
        try {
            soleList = saleOutDao.getU8SaleOut(entity);
        } catch (Exception e) {
            logger.info("销售出库单查询失败"+e);
            logger.info("销售出库单查询失败1"+e.getMessage());
            throw new RuntimeException(e);
        }
        logger.info("销售出库查询数据{}",JSONObject.toJSONString(soleList));
        if(CollectionUtils.isNotEmpty(soleList)){
            soleList.forEach(sale -> {
                try{
                    //获取明细数据进行拼接
                    String id = sale.getId();
                    if(!StrUtil.isEmpty(id)){
                        SaleOutDetailEntity line = new SaleOutDetailEntity();
                        line.setId(id);
                        line.setAccId(sale.getAccId());
                        line.setDataSourceCode(requestJson.getString("db_code"));
                        List<SaleOutDetailEntity> lineList = saleOutDetailDao.getU8SaleOutLine(line);
                        logger.info("销售出库单明细查询数据{}",JSONObject.toJSONString(lineList));
                        sale.setSaleOutDetailEntityList(lineList);
                        logger.info("组装数据");
                        JSONObject main = bindingAdd(sale);
                        logger.info("销售出库推送数据："+main.toString());
                        String result = CrmUtil.sendU8TOCrmEsb(main.toString(), "8000070012");
                        logger.info("销售出库推送结果："+result);
                        IntegrationTaskLivingDetailsEntity logDetails = new IntegrationTaskLivingDetailsEntity();
                        logDetails.setRootAppPk(sale.getIds());
                        logDetails.setRootAppBill(sale.getCcode());
                        logDetails.setNewTransmitInfo(result);
                        logDetails.setNewPushDate(new Date());
                        logDetails.setRootAppNewData(JSON.toJSONString(sale));
                        logDetails.setPluginId("SaleOutPluginInitializer");
                        logger.info("111111111111111111111111");

                        JSONObject resultObj = JSON.parseObject(result);
                        boolean flag = resultObj.getString("code").equals("200");
                        if(flag){
                            logDetails.setNewTransmitInfo("推送成功");
                            // 推送成功，更新状态
                            sale.setState("Y");
                        }else{
                            // 推送失败
                            sale.setState("N");
                        }
                         logger.info("222222222222222222");
                        sale.setDataSourceCode(requestJson.getString("db_code"));
                        try{
                            saveLog(interId, logDetails, flag);
                        }catch (Exception e){
                            logger.error("保存日志失败"+e.getMessage());
                            saleOutDao.updateState(sale);
                            e.printStackTrace();
                        }
                         logger.info("3333333333333333333333");
                        saleOutDao.updateState(sale);
                    }
                } catch (Exception e) {
                    logger.error("销售出库执行失败"+e.getMessage());
                    e.printStackTrace();
                }
            });

        }
    }


    /**
 *
 * @content 保存日志信息
 * @className: Administrator
 * @author laborer
 * @date 2024-10-18 11:27
 *
 */

    private  void saveLog(String interId, IntegrationTaskLivingDetailsEntity logDetails, boolean flag) throws Exception {
        if(StrUtil.isEmpty(interId)){
            if(flag){
                //首推成功
                taskLivingDetailsService.saveLogToSuccess(logDetails);
            }else{
                //首推失败
                taskLivingDetailsService.saveLogToFail(logDetails);
            }
        }else{
            logDetails.setId(interId);
            if(flag){
                //重推成功
                taskLivingDetailsService.saveLogFailToSuccess(logDetails);
            }else{
                //重推失败，把失败信息，更新到重推表中
                taskLivingDetailsService.updateLogFailToSuccess(logDetails);
            }
        }
    }

    /**
 *
 * @content 组装推送参数
 * @className: Administrator
 * @author laborer
 * @date 2024-10-18 11:13
 *
 */

    private JSONObject bindingAdd(SaleOutEntity sale) {
        JSONObject main = new JSONObject();
        main.put("AccId",sale.getAccId());
        main.put("billid",sale.getId());
        main.put("LoginDate", DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss"));
        JSONObject head = new JSONObject();
        head.put("cdlcode",sale.getCdlcode());
        head.put("couttype",sale.getOuttype());
        head.put("ccode",sale.getCcode());
        head.put("ddate",sale.getDdate());
        JSONArray body = new JSONArray();
        List<SaleOutDetailEntity> lineList = sale.getSaleOutDetailEntityList();
        if(CollectionUtils.isNotEmpty(lineList)){
            lineList.forEach(line -> {
                JSONObject lineObj = new JSONObject();
                lineObj.put("cinvcode",line.getCinvcode());
                lineObj.put("cinvname",line.getCinvname());
                lineObj.put("iquantity",line.getIquantity());
                lineObj.put("inum",line.getInum());
                lineObj.put("idlrowno",line.getIdlrowno());
                body.add(lineObj);
            });
        }
        main.put("head",head);
        main.put("body",body);
        return main;
    }








    /**
     *
     * @content 销售出库-新增
     * @className: Administrator
     * @author laborer
     * @date 2025-02-26 14:39
     *
     */

    @Override
    public U8ResultEntity addSaleOut(JSONObject parm) {
        String billCode = "";
        String dlId = "";
        String AccId = "";
        JSONObject jsonObject = parm.getJSONObject("jsonStr");
        logger.info("接收到的参数"+jsonObject.toString());
        StringBuffer resultStr = new StringBuffer();//回传json
        try {
            String billid = jsonObject.getString("billid");
            if(StrUtil.isEmpty(billid)){
                return BaseU8Result.getFailureMessageEntity("请传入第三方唯一标识");
            }
//            AccId = jsonObject.getString("AccId");
//            if(StrUtil.isEmpty(AccId)){
//                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
//            }
//            String AccCode = U8Util.accMap.get(AccId);
//            if(StrUtil.isEmpty(AccCode)){
//                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
//            }
            JSONObject jHead = jsonObject.getJSONObject("head");
            //todo 发货单表头必输项
            String[] MustInputCode = new String[] {  "cbuscode", "crdcode", "cwhcode", "cmaker"};
            String[] MustInputName = new String[] {  "发货单号", "收发类别", "仓库", "制单人"};
            //根据当前单据类型 所需参数判断是否传入该参数没有传入
            for (int i = 0; i < MustInputCode.length; i++)
            {
                if (StrUtil.isEmpty(jHead.getString(MustInputCode[i]))) {
                    resultStr.append("请输入：" + MustInputCode[i] + "(" + MustInputName[i] + ")") ;
                }
            }
            //如果有错误信息，则返回错误信息
            if(StrUtil.isNotEmpty(resultStr.toString())){
                return BaseU8Result.getFailureMessageEntity(resultStr.toString());
            }
            String sqlAccId = "select * from [kjs_external].[dbo].[v_dispatchlist_forout] where cDLCode = '"+jHead.getString("cbuscode")+"'";
            //查询账套中的发货单信息
            SaleInvEntity U8Obj = new SaleInvEntity();
            U8Obj.setDataSourceCode("YONYOUU8NEW");
            List<HashMap<String, Object>> hashMaps = saleInvDao.getU8BillData(sqlAccId,U8Obj);
            if(CollectionUtils.isEmpty(hashMaps) || hashMaps.size() > 1){
                return BaseU8Result.getFailureMessageEntity("发货单不存在，或存在多个发货单号为："+jHead.getString("cbuscode"));
            }
            HashMap<String, Object> cbusc= hashMaps.get(0);
             AccId = String.valueOf(cbusc.get("zhangtao"));
             if(StrUtil.isEmpty(AccId)){
                 return BaseU8Result.getFailureMessageEntity("获取账套号有误，请联系管理员"+AccId);
             }
            String AccCode = U8Util.accMap.get(AccId);
            if(StrUtil.isEmpty(AccCode)){
                return BaseU8Result.getFailureMessageEntity("获取账套号有误，请联系管理员");
            }

            //验证单号唯一
            String cdefine5 = jsonObject.getString("cdefine5");
            if(StrUtil.isNotEmpty(cdefine5)){
                String cdefine5Sql = "select cdefine5 from "+AccCode+".dbo.rdrecord32 where cdefine5 = '"+cdefine5+"'";
                List<HashMap<String, Object>> cdefine5Map = saleInvDao.getU8BillData(cdefine5Sql,U8Obj);
                if(CollectionUtils.isNotEmpty(cdefine5Map) ){
                    return BaseU8Result.getFailureMessageEntity("单号重复,请检查："+cdefine5);
                }
            }

            //发货退货单主表@
            //发货退货单主表
            String sqlSelDisList = "select DLID as 'dlid',DLID as 'cdlcode' ,cDLCode as 'cbuscode' ,cDepCode as 'cdepcode' ,cPersonCode as 'cpersoncode' ,cCusCode as 'ccuscode'," +
                    "cCusCode as 'cinvoicecompany',cSTCode as 'cstcode' ,'32' as 'cvouchtype' ,'普通销售' as 'cbustype' ,'发货单' as 'csource' ,GETDATE() as 'ddate' ,'form WMS' as 'cmemo'," +
                    "case when bReturnFlag = 1 then '多组织-红字采购入库' else '多组织-采购订单' end as 'cdefine1' " +
                    "from "+AccCode+".dbo.DispatchList " +
                    "where cDLCode = '"+jHead.getString("cbuscode")+"'";

            List<HashMap<String, Object>> selDisListMap = saleInvDao.getU8BillData(sqlSelDisList,U8Obj);
            if(CollectionUtils.isEmpty(selDisListMap) ){
                return BaseU8Result.getFailureMessageEntity("发货单主表查询失败"+jHead.getString("cbuscode"));
            }
            HashMap<String, Object> selDis= hashMaps.get(0);
            if(StrUtil.isEmpty(jHead.getString("dlid"))){
                jHead.put("dlid",selDis.get("dlid"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdlcode"))){
                jHead.put("cdlcode",selDis.get("cdlcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbuscode"))){
                jHead.put("cbuscode",selDis.get("cbuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdepcode"))){
                jHead.put("cdepcode",selDis.get("cdepcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cpersoncode"))){
                jHead.put("cpersoncode",selDis.get("cpersoncode"));
            }
            if(StrUtil.isEmpty(jHead.getString("ccuscode"))){
                jHead.put("ccuscode",selDis.get("ccuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cvouchtype"))){
                jHead.put("cvouchtype",selDis.get("cvouchtype"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbustype"))){
                jHead.put("cbustype",selDis.get("cbustype"));
            }
            if(StrUtil.isEmpty(jHead.getString("csource"))){
                jHead.put("csource",selDis.get("csource"));
            }
            if(StrUtil.isEmpty(jHead.getString("ddate"))){
                jHead.put("ddate",selDis.get("ddate"));
            }
            if(StrUtil.isEmpty(jHead.getString("cmemo"))){
                jHead.put("cmemo",selDis.get("cmemo"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdefine1"))){
                jHead.put("cdefine1",selDis.get("cdefine1"));
            }
            //表体
            JSONArray body = new JSONArray();
            JSONArray jBody = jsonObject.getJSONArray("body");
            for (int i = 0; i < jBody.size(); i++) {
                JSONObject jBodyObj = jBody.getJSONObject(i);
                if(StrUtil.isEmpty(jBodyObj.getString("cinvcode"))){
                    resultStr.append("材料编码不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if(StrUtil.isEmpty(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if("0".equals(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为0 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                //发货退货单子表@
                String sqlSelDisLists = " select idlsid  from  "+AccCode+".dbo.DispatchList dl  left join  "+AccCode+".dbo.DispatchLists dls on dls.DLID = dl.DLID  " +
                        "where cDLCode = '"+jBodyObj.getString("cbdlcode")+"" +
                        "' and cInvCode = '"+jBodyObj.getString("cinvcode")+"" +
                        "' and irowno = '"+jBodyObj.getString("cdblrowno")+"'";
                List<HashMap<String, Object>> selDisDetailsListMap = saleInvDao.getU8BillData(sqlSelDisLists,U8Obj);
                if(CollectionUtils.isEmpty(selDisDetailsListMap) ){
                    return BaseU8Result.getFailureMessageEntity("发货单子表查询失败"+jHead.getString("cbuscode")+"发货单表体存货"+jBodyObj.getString("cinvcode")+"行号"+jBodyObj.getString("cdblrowno")+"不存在");
                }
                HashMap<String, Object> selDisDetails= hashMaps.get(0);
                if(StrUtil.isEmpty(jBodyObj.getString("idlsid"))){
                    jBodyObj.put("idlsid",selDisDetails.get("idlsid"));
                }


                //获取辅计量单位
                String dtInvSql = "select cInvCode,cInvName,cInvStd,Inv.cComUnitCode,cInvCCode,iChangRate, cPUComUnitCode,cSAComUnitCode,cProductUnit,cSTComUnitCode " +
                        "from "+AccCode+".dbo.Inventory Inv " +
                        "left join "+AccCode+".dbo.ComputationUnit Com On Inv.cComUnitCode = Com.cComUnitCode " +
                        "where cInvCode='" + jBodyObj.getString("cinvcode") +"'";
                List<HashMap<String, Object>> dtInv = saleInvDao.getU8BillData(dtInvSql,U8Obj);
                if(CollectionUtils.isEmpty(dtInv) ){
                    return BaseU8Result.getFailureMessageEntity("ERP没有该物料，请检查，行号："+i+1);
                }
                HashMap<String, Object> dtInvObj= dtInv.get(0);
                //存在辅计量单位则 计算件数 =数量/换算率
                if(StrUtil.isNotEmpty(String.valueOf(dtInvObj.get("cSTComUnitCode")))){
                    jBodyObj.put("cassunit",dtInvObj.get("cSTComUnitCode"));
                    String sqlCassUnit = "select * from "+AccCode+".dbo.ComputationUnit where cComUnitCode='" +dtInvObj.get("cSTComUnitCode")+"'";
                    List<HashMap<String, Object>> dtUnit = saleInvDao.getU8BillData(sqlCassUnit,U8Obj);
                    if(CollectionUtils.isEmpty(dtUnit) ){
                        return BaseU8Result.getFailureMessageEntity("查询单位失败，请检查，行号："+i+1);
                    }
                    HashMap<String, Object> dtUnitObj= dtUnit.get(0);
                    jBodyObj.put("iinvexchrate",dtUnitObj.get("iChangRate"));

                    jBodyObj.put("inum",jBodyObj.getBigDecimal("iquantity").divide(new BigDecimal(String.valueOf(dtUnitObj.get("iChangRate"))),6, RoundingMode.HALF_UP));

                }
                jBodyObj.put("irowno",i+1);
                //默认字段
                //是否需要开票
                if(StrUtil.isEmpty(jBodyObj.getString("bneedbill"))){
                    jBodyObj.put("bneedbill",1);
                }
                //是否存货核算开票
                if(StrUtil.isEmpty(jBodyObj.getString("bIAcreatebill"))){
                    jBodyObj.put("bIAcreatebill",0);
                }
                //是否销售出库开票
                if(StrUtil.isEmpty(jBodyObj.getString("bsaleoutcreatebill"))){
                    jBodyObj.put("bsaleoutcreatebill",0);
                }
                //存货是05分类下的且没有供应商存货调价单就报错
                if(!"001".equals(AccId)){
                    String sqlInv = " select cInvCCode from "+AccCode+".dbo.Inventory where cInvCode = '"+jBodyObj.getString("cinvcode")+"' and cInvCCode like '05%' ";
                    List<HashMap<String, Object>> dtInvSu = saleInvDao.getU8BillData(sqlInv,U8Obj);
                    if(CollectionUtils.isNotEmpty(dtInvSu) ){
                        String sqlVenPrice = "select * " +
                                "from  "+AccCode+".dbo.Ven_Inv_Price " +
                                "where cVenCode = '001' and cInvCode = '"+jBodyObj.getString("cinvcode")+"' " +
                            "and dEnableDate <= convert(varchar,getdate(),23) " +
                            "and ISNULL(dDisableDate,'9999-12-31') > convert(varchar,getdate(),23) " +
                            "order by dEnableDate ";
                        List<HashMap<String, Object>> dtVenPrice = saleInvDao.getU8BillData(sqlVenPrice,U8Obj);
                        if(CollectionUtils.isEmpty(dtVenPrice)){
                            return BaseU8Result.getFailureMessageEntity("存货："+jBodyObj.getString("cinvcode")+"供应商：001，供应商存货调价单不存在！");
                        }
                    }
                }
                body.add(jBodyObj);
            }
            JSONObject main = new JSONObject();
            main.put("head",jHead);
            main.put("billid",billid);
            main.put("AccId",AccId);
            main.put("head",jHead);
            main.put("body",body);
            main.put("domPosition",new ArrayList<>());
            logger.info("销售出库单新增推送U8："+main.toString());
            String result = U8Util.sengBillToU8(main.toString(), "8000110021");
            logger.info("销售出库单新增推送U8结果："+result);
            JSONObject resultObj = JSON.parseObject(result);
            boolean flag = resultObj.getBoolean("success");
            logger.info(resultObj.toString());
            if(!flag){
                return BaseU8Result.getFailureMessageEntity("新增失败：：："+resultObj.getString("message"));
            }else{
                billCode = resultObj.getString("message");
                //审核
                JSONObject audit = new JSONObject();
                audit.put("billid", UUID.randomUUID());
                audit.put("AccId", AccId);
                audit.put("cVoucherId", billCode);
                logger.info("销售出库单审核："+main.toString());
                String resultAudit = U8Util.sengBillToU8(audit.toString(), "8000110022");
                logger.info("销售出库单审核结果："+result);
                JSONObject resultObjA = JSON.parseObject(resultAudit);
                boolean flagAudit = resultObjA.getBoolean("success");
//                logger.info(resultObj.toString());
                if(!flagAudit){
                    //如果审核失败需要调用删除接口回滚单据
                    JSONObject delete = new JSONObject();
                    delete.put("billid", UUID.randomUUID());
                    delete.put("AccId", AccId);
                    delete.put("cVoucherId", billCode);
                    //删除
                    String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
                    //删除之后直接返回失败信息
                    return BaseU8Result.getFailureMessageEntity("新增正常但审核失败：：："+resultObjA.getString("message"));
                }
            }
            return BaseU8Result.getSuccessMessageEntity(resultObj.getString("message"));
        }  catch (Exception e) {
            //如果审核失败需要调用删除接口回滚单据
            JSONObject delete = new JSONObject();
            delete.put("billid", UUID.randomUUID());
            delete.put("AccId", AccId);
            delete.put("cVoucherId", billCode);
            //删除
            String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
            logger.info("新增失败："+e.getMessage());
            return BaseU8Result.getFailureMessageEntity("新增失败，请联系U8管理员");
        }
    }

    @Override
    public U8ResultEntity addSaleOutT(JSONObject jsonObject) {
        String billCode = "";
        String dlId = "";
        String AccId = "";
//        JSONObject jsonObject = parm.getJSONObject("jsonStr");
        logger.info("接收到的参数"+jsonObject.toString());
        StringBuffer resultStr = new StringBuffer();//回传json
        try {
            String billid = jsonObject.getString("billid");
            if(StrUtil.isEmpty(billid)){
                return BaseU8Result.getFailureMessageEntity("请传入第三方唯一标识");
            }
//            AccId = jsonObject.getString("AccId");
//            if(StrUtil.isEmpty(AccId)){
//                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
//            }
//            String AccCode = U8Util.accMap.get(AccId);
//            if(StrUtil.isEmpty(AccCode)){
//                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
//            }
            JSONObject jHead = jsonObject.getJSONObject("head");
            //todo 发货单表头必输项
            String[] MustInputCode = new String[] {  "cbuscode", "crdcode", "cwhcode", "cmaker"};
            String[] MustInputName = new String[] {  "发货单号", "收发类别", "仓库", "制单人"};
            //根据当前单据类型 所需参数判断是否传入该参数没有传入
            for (int i = 0; i < MustInputCode.length; i++)
            {
                if (StrUtil.isEmpty(jHead.getString(MustInputCode[i]))) {
                    resultStr.append("请输入：" + MustInputCode[i] + "(" + MustInputName[i] + ")") ;
                }
            }
            //如果有错误信息，则返回错误信息
            if(StrUtil.isNotEmpty(resultStr.toString())){
                return BaseU8Result.getFailureMessageEntity(resultStr.toString());
            }
            String sqlAccId = "select * from [kjs_external].[dbo].[v_dispatchlist_forout] where cDLCode = '"+jHead.getString("cbuscode")+"'";
            //查询账套中的发货单信息
            SaleInvEntity U8Obj = new SaleInvEntity();
            U8Obj.setDataSourceCode("YONYOUU8NEW");
            List<HashMap<String, Object>> hashMaps = saleInvDao.getU8BillData(sqlAccId,U8Obj);
            if(CollectionUtils.isEmpty(hashMaps) || hashMaps.size() > 1){
                return BaseU8Result.getFailureMessageEntity("发货单不存在，或存在多个发货单号为："+jHead.getString("cbuscode"));
            }
            HashMap<String, Object> cbusc= hashMaps.get(0);
            AccId = String.valueOf(cbusc.get("zhangtao"));
            if(StrUtil.isEmpty(AccId)){
                return BaseU8Result.getFailureMessageEntity("获取账套号有误，请联系管理员"+AccId);
            }
            String AccCode = U8Util.accMap.get(AccId);
            if(StrUtil.isEmpty(AccCode)){
                return BaseU8Result.getFailureMessageEntity("获取账套号有误，请联系管理员");
            }
            String ccodeSql = "SELECT ISNULL(max(CONVERT(DECIMAL(20, 0), ccode))+1, CONVERT (VARCHAR (20),ddate,112) + '0001') as ccode " +
                    "FROM "+AccCode+".dbo.RdRecord32 " +
                    "WHERE cCode LIKE CONVERT (VARCHAR (20),'"+DateUtil.format(jHead.getDate("ddate"),"yyyyMMdd")+"',112) + '%'" +
                    "AND len(cCode) = 12 GROUP BY ddate";
            //查询账套中的发货单信息
//            SaleInvEntity U8Obj = new SaleInvEntity();
//            U8Obj.setDataSourceCode("YONYOUU8NEW");
            List<HashMap<String, Object>> ccodeHashMaps = saleInvDao.getU8BillData(ccodeSql,U8Obj);
            if(CollectionUtils.isNotEmpty(ccodeHashMaps)){
                HashMap<String, Object> ccodeObj= ccodeHashMaps.get(0);
                jHead.put("ccode",ccodeObj.get("ccode"));
            }else{
                jHead.put("ccode",DateUtil.format(jHead.getDate("ddate"),"yyyyMMdd")+"0001");
            }
            jHead.put("csource","发货单");



            //验证单号唯一
//            String cdefine5 = jsonObject.getString("cdefine5");
//            if(StrUtil.isNotEmpty(cdefine5)){
//                String cdefine5Sql = "select cdefine5 from "+AccCode+".dbo.rdrecord32 where cdefine5 = '"+cdefine5+"'";
//                List<HashMap<String, Object>> cdefine5Map = saleInvDao.getU8BillData(cdefine5Sql,U8Obj);
//                if(CollectionUtils.isNotEmpty(cdefine5Map) ){
//                    return BaseU8Result.getFailureMessageEntity("单号重复,请检查："+cdefine5);
//                }
//            }

            //发货退货单主表@
            //发货退货单主表
            String sqlSelDisList = "select DLID as 'dlid',DLID as 'cdlcode' ,cDLCode as 'cbuscode' ,cDepCode as 'cdepcode' ,cPersonCode as 'cpersoncode' ,cCusCode as 'ccuscode'," +
                    "cCusCode as 'cinvoicecompany',cSTCode as 'cstcode' ,'32' as 'cvouchtype' ,'普通销售' as 'cbustype' ,'发货单' as 'csource' ,GETDATE() as 'ddate' ,'form WMS' as 'cmemo'," +
                    "case when bReturnFlag = 1 then '多组织-红字采购入库' else '多组织-采购订单' end as 'cdefine1' " +
                    "from "+AccCode+".dbo.DispatchList " +
                    "where cDLCode = '"+jHead.getString("cbuscode")+"'";

            List<HashMap<String, Object>> selDisListMap = saleInvDao.getU8BillData(sqlSelDisList,U8Obj);
            if(CollectionUtils.isEmpty(selDisListMap) ){
                return BaseU8Result.getFailureMessageEntity("发货单主表查询失败"+jHead.getString("cbuscode"));
            }
            HashMap<String, Object> selDis= hashMaps.get(0);
            if(StrUtil.isEmpty(jHead.getString("dlid"))){
                jHead.put("dlid",selDis.get("dlid"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdlcode"))){
                jHead.put("cdlcode",selDis.get("cdlcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbuscode"))){
                jHead.put("cbuscode",selDis.get("cbuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdepcode"))){
                jHead.put("cdepcode",selDis.get("cdepcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cpersoncode"))){
                jHead.put("cpersoncode",selDis.get("cpersoncode"));
            }
            if(StrUtil.isEmpty(jHead.getString("ccuscode"))){
                jHead.put("ccuscode",selDis.get("ccuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cvouchtype"))){
                jHead.put("cvouchtype",selDis.get("cvouchtype"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbustype"))){
                jHead.put("cbustype",selDis.get("cbustype"));
            }
            if(StrUtil.isEmpty(jHead.getString("csource"))){
                jHead.put("csource",selDis.get("csource"));
            }
            if(StrUtil.isEmpty(jHead.getString("ddate"))){
                jHead.put("ddate",selDis.get("ddate"));
            }
            if(StrUtil.isEmpty(jHead.getString("cmemo"))){
                jHead.put("cmemo",selDis.get("cmemo"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdefine1"))){
                jHead.put("cdefine1",selDis.get("cdefine1"));
            }
            //表体
            JSONArray body = new JSONArray();
            JSONArray jBody = jsonObject.getJSONArray("body");
            int k = 1;
            for (int i = 0; i < jBody.size(); i++) {
                System.out.println("错误的行"+k);
                k++;
                JSONObject jBodyObj = jBody.getJSONObject(i);
                if(StrUtil.isEmpty(jBodyObj.getString("cinvcode"))){
                    resultStr.append("材料编码不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if(StrUtil.isEmpty(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if("0".equals(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为0 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                //发货退货单子表@
                String sqlSelDisLists = " select idlsid,irowno  from  "+AccCode+".dbo.DispatchList dl  left join  "+AccCode+".dbo.DispatchLists dls on dls.DLID = dl.DLID  " +
                        "where cDLCode = '"+jBodyObj.getString("cbdlcode")+"" +
                        "' and cInvCode = '"+jBodyObj.getString("cinvcode")+"'" ;
//                        "' and irowno = '"+jBodyObj.getString("cdblrowno")+"'";
                List<HashMap<String, Object>> selDisDetailsListMap = saleInvDao.getU8BillData(sqlSelDisLists,U8Obj);
                if(CollectionUtils.isEmpty(selDisDetailsListMap)&& selDisDetailsListMap.size()==0){
                    continue;
//                    return BaseU8Result.getFailureMessageEntity("发货单子表查询失败"+jHead.getString("cbuscode")+"发货单表体存货"+jBodyObj.getString("cinvcode")+"行号"+jBodyObj.getString("cdblrowno")+"不存在");
                }
                HashMap<String, Object> selDisDetails= selDisDetailsListMap.get(0);
                if(StrUtil.isEmpty(jBodyObj.getString("idlsid"))){
                    jBodyObj.put("idlsid",selDisDetails.get("idlsid"));
                    jBodyObj.put("irowno",selDisDetails.get("irowno"));
                }


                //获取辅计量单位
                String dtInvSql = "select inv.bInvBatch,cInvCode,cInvName,cInvStd,Inv.cComUnitCode,cInvCCode,iChangRate, cPUComUnitCode,cSAComUnitCode,cProductUnit,cSTComUnitCode " +
                        "from "+AccCode+".dbo.Inventory Inv " +
                        "left join "+AccCode+".dbo.ComputationUnit Com On Inv.cComUnitCode = Com.cComUnitCode " +
                        "where cInvCode='" + jBodyObj.getString("cinvcode") +"'";
                List<HashMap<String, Object>> dtInv = saleInvDao.getU8BillData(dtInvSql,U8Obj);
                if(CollectionUtils.isEmpty(dtInv) ){
                    return BaseU8Result.getFailureMessageEntity("ERP没有该物料，请检查，行号："+i+1);
                }
                HashMap<String, Object> dtInvObj= dtInv.get(0);
                //存在辅计量单位则 计算件数 =数量/换算率
                if(StrUtil.isNotEmpty(String.valueOf(dtInvObj.get("cSTComUnitCode")))){
                    jBodyObj.put("cassunit",dtInvObj.get("cSTComUnitCode"));
                    String sqlCassUnit = "select * from "+AccCode+".dbo.ComputationUnit where cComUnitCode='" +dtInvObj.get("cSTComUnitCode")+"'";
                    List<HashMap<String, Object>> dtUnit = saleInvDao.getU8BillData(sqlCassUnit,U8Obj);
                    if(CollectionUtils.isEmpty(dtUnit) ){
//                        return BaseU8Result.getFailureMessageEntity("查询单位失败，请检查，行号："+i+1);
                        jBodyObj.put("iinvexchrate",0);
                    }else {
                        HashMap<String, Object> dtUnitObj= dtUnit.get(0);
                        jBodyObj.put("iinvexchrate",dtUnitObj.get("iChangRate"));
                        jBodyObj.put("inum",jBodyObj.getBigDecimal("iquantity").divide(jBodyObj.getBigDecimal("iinvexchrate"),6, RoundingMode.HALF_UP));
                    }




                }

                //默认字段
                //是否需要开票
                if(StrUtil.isEmpty(jBodyObj.getString("bneedbill"))){
                    jBodyObj.put("bneedbill",1);
                }
                //是否存货核算开票
                if(StrUtil.isEmpty(jBodyObj.getString("bIAcreatebill"))){
                    jBodyObj.put("bIAcreatebill",0);
                }
                //是否销售出库开票
                if(StrUtil.isEmpty(jBodyObj.getString("bsaleoutcreatebill"))){
                    jBodyObj.put("bsaleoutcreatebill",0);
                }
                //存货是05分类下的且没有供应商存货调价单就报错
//                if(!"001".equals(AccId)){
//                    String sqlInv = " select cInvCCode from "+AccCode+".dbo.Inventory where cInvCode = '"+jBodyObj.getString("cinvcode")+"' and cInvCCode like '05%' ";
//                    List<HashMap<String, Object>> dtInvSu = saleInvDao.getU8BillData(sqlInv,U8Obj);
//                    if(CollectionUtils.isNotEmpty(dtInvSu) ){
//                        String sqlVenPrice = "select * " +
//                                "from  "+AccCode+".dbo.Ven_Inv_Price " +
//                                "where cVenCode = '001' and cInvCode = '"+jBodyObj.getString("cinvcode")+"' " +
//                                "and dEnableDate <= convert(varchar,getdate(),23) " +
//                                "and ISNULL(dDisableDate,'9999-12-31') > convert(varchar,getdate(),23) " +
//                                "order by dEnableDate ";
//                        List<HashMap<String, Object>> dtVenPrice = saleInvDao.getU8BillData(sqlVenPrice,U8Obj);
//                        if(CollectionUtils.isEmpty(dtVenPrice)){
//                            return BaseU8Result.getFailureMessageEntity("存货："+jBodyObj.getString("cinvcode")+"供应商：001，供应商存货调价单不存在！");
//                        }
//                    }
//                }
                jBodyObj.put("cProBatch",0);
//                jBodyObj.put("csource","发货单");
//                jBodyObj.put("cbustype","普通销售");
                jBodyObj.put("cProBatch",0);
                String bInvBatch = String.valueOf(dtInvObj.get("bInvBatch"));
                if("true".equals(bInvBatch)){
                    jBodyObj.put("cbatch",0);
                }

                body.add(jBodyObj);
            }
            JSONObject main = new JSONObject();
            main.put("head",jHead);
            main.put("billid",billid);
            main.put("AccId",AccId);
            main.put("head",jHead);
            main.put("body",body);
            main.put("domPosition",new ArrayList<>());
            logger.info("销售出库单新增推送U8："+main.toString());
            String result = U8Util.sengBillToU8(main.toString(), "8000110021");
            logger.info("销售出库单新增推送U8结果："+result);
            JSONObject resultObj = JSON.parseObject(result);
            boolean flag = resultObj.getBoolean("success");
            logger.info(resultObj.toString());
            if(!flag){
                return BaseU8Result.getFailureMessageEntity("新增失败：：："+resultObj.getString("message"));
            }else{
                billCode = resultObj.getString("message");
                //审核
                JSONObject audit = new JSONObject();
                audit.put("billid", UUID.randomUUID());
                audit.put("AccId", AccId);
                audit.put("cVoucherId", billCode);
                logger.info("销售出库单审核："+main.toString());
                String resultAudit = U8Util.sengBillToU8(audit.toString(), "8000110022");
                logger.info("销售出库单审核结果："+result);
                JSONObject resultObjA = JSON.parseObject(resultAudit);
                boolean flagAudit = resultObjA.getBoolean("success");
//                logger.info(resultObj.toString());
                if(!flagAudit){
                    //如果审核失败需要调用删除接口回滚单据
                    JSONObject delete = new JSONObject();
                    delete.put("billid", UUID.randomUUID());
                    delete.put("AccId", AccId);
                    delete.put("cVoucherId", billCode);
                    //删除
                    String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
                    //删除之后直接返回失败信息
                    return BaseU8Result.getFailureMessageEntity("新增正常但审核失败：：："+resultObjA.getString("message"));
                }
            }
            return BaseU8Result.getSuccessMessageEntity(resultObj.getString("message"));
        }  catch (Exception e) {
            //如果审核失败需要调用删除接口回滚单据
            JSONObject delete = new JSONObject();
            delete.put("billid", UUID.randomUUID());
            delete.put("AccId", AccId);
            delete.put("cVoucherId", billCode);
            if(StrUtil.isNotEmpty(billCode)){
                String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
            }
            //删除
            logger.info("新增失败："+e.getMessage());
            return BaseU8Result.getFailureMessageEntity("新增失败，请联系U8管理员");
        }
    }

    @Override
    public U8ResultEntity deleteSaleOut(JSONObject parm) {
        String billCode = "";
        String dlId = "";
        String AccId = "";
        JSONObject jsonObject = parm.getJSONObject("jsonStr");
        JSONObject jHead = jsonObject.getJSONObject("head");
        logger.info("接收到的参数"+jsonObject.toString());
        StringBuffer resultStr = new StringBuffer();//回传json
        try {
            String billid = jsonObject.getString("billid");
            if(StrUtil.isEmpty(billid)){
                return BaseU8Result.getFailureMessageEntity("请传入第三方唯一标识");
            }
            AccId = jsonObject.getString("AccId");
            if(StrUtil.isEmpty(AccId)){
                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
            }
            String AccCode = U8Util.accMap.get(AccId);
            if(StrUtil.isEmpty(AccCode)){
                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
            }
            String cVoucherId = jHead.getString("cVoucherId");
            if(StrUtil.isEmpty(cVoucherId)){
                return BaseU8Result.getFailureMessageEntity("销售出库单删除异常：cVoucherId 单据id 未输入");
            }
            //弃审
            String isUnAudit = jHead.getString("isUnAudit");
            if(StrUtil.isNotEmpty(isUnAudit) && isUnAudit.equals("1")){
                String sql = " select cHandler as 'auditPersonName' from "+AccCode+".dbo.rdrecord32 where ID = '"+cVoucherId+"' ";
                SaleInvEntity erdt = new SaleInvEntity();
                erdt.setDataSourceCode("YONYOUU8NEW");
                List<HashMap<String, Object>> erdtMap = saleInvDao.getU8BillData(sql,erdt);
                if(CollectionUtils.isEmpty(erdtMap)){
                    return BaseU8Result.getFailureMessageEntity("销售出库单删除-销售出库单不存在或已被删除！"+cVoucherId);
                }
                HashMap<String, Object> dt = erdtMap.get(0);
                String auditPersonName = String.valueOf(dt.get("auditPersonName"));
                if(StrUtil.isNotEmpty(auditPersonName)){
                    JSONObject json = new JSONObject();
                    json.put("Token", "Hzya1314_CheckSkip");
                    json.put("billid", cVoucherId);
                    json.put("AccId", AccId);
                    json.put("cVoucherId", cVoucherId);
                    //弃审
                    String resultUnAudit = U8Util.sengBillToU8(json.toString(), "8000110024");
                    JSONObject resultObj = JSON.parseObject(resultUnAudit);
                    boolean flag = resultObj.getBoolean("success");
                    if(!flag){
                        return BaseU8Result.getFailureMessageEntity("销售出库单自动弃审失败："+cVoucherId);
                    }else{
                        //删除
                        JSONObject delete = new JSONObject();
                        delete.put("Token", "Hzya1314_CheckSkip");
                        delete.put("billid", cVoucherId);
                        delete.put("AccId", AccId);
                        delete.put("cVoucherId", cVoucherId);
                        String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
                        JSONObject resultDeleteObj = JSON.parseObject(resultDelete);
                        boolean deleteFlag = resultDeleteObj.getBoolean("success");
                        if(deleteFlag){
                            return BaseU8Result.getSuccessMessageEntity("销售出库单删除成功");
                        }else{
                            return BaseU8Result.getFailureMessageEntity("销售出库单删除失败："+resultDelete);
                        }
                    }

                }
            }
        }  catch (Exception e) {
            return BaseU8Result.getFailureMessageEntity("销售出库单删除失败，请联系U8管理员");
        }
        return BaseU8Result.getSuccessMessageEntity("销售出库单删除成功");
    }


    /// <summary>
    /// 销售出库单-新增（发货单部分作废用）
    /// </summary>
    /// <param name="json"></param>
    /// <returns></returns>
    public U8ResultEntity Add_DLPartialClose(JSONObject parm) {
        String billCode = "";
        String dlId = "";
        String AccId = "";
        JSONObject jsonObject = parm.getJSONObject("jsonStr");
        logger.info("接收到的参数"+jsonObject.toString());
        StringBuffer resultStr = new StringBuffer();//回传json
        try {
            String billid = jsonObject.getString("billid");
            if(StrUtil.isEmpty(billid)){
                return BaseU8Result.getFailureMessageEntity("请传入第三方唯一标识");
            }
            AccId = jsonObject.getString("AccId");
            if(StrUtil.isEmpty(AccId)){
                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
            }
            String AccCode = U8Util.accMap.get(AccId);
            if(StrUtil.isEmpty(AccCode)){
                return BaseU8Result.getFailureMessageEntity("请传入正确的账套号");
            }
            JSONObject jHead = jsonObject.getJSONObject("head");
            //todo 发货单表头必输项
            String[] MustInputCode = new String[] {  "cbuscode", "crdcode", "cwhcode", "cmaker"};
            String[] MustInputName = new String[] {  "发货单号", "收发类别", "仓库", "制单人"};
            //根据当前单据类型 所需参数判断是否传入该参数没有传入
            for (int i = 0; i < MustInputCode.length; i++)
            {
                if (StrUtil.isEmpty(jHead.getString(MustInputCode[i]))) {
                    resultStr.append("请输入：" + MustInputCode[i] + "(" + MustInputName[i] + ")") ;
                }
            }
            //如果有错误信息，则返回错误信息
            if(StrUtil.isNotEmpty(resultStr.toString())){
                return BaseU8Result.getFailureMessageEntity(resultStr.toString());
            }
            //发货退货单主表
            String sqlSelDisList = "select DLID as 'dlid' ,DLID as 'cdlcode' ,cDLCode as 'cbuscode' ,cDepCode as 'cdepcode' ," +
                    "cPersonCode as 'cpersoncode' ,cCusCode as 'ccuscode' ,cCusCode as 'cinvoicecompany' ,cSTCode as 'cstcode' ," +
                    "'32' as 'cvouchtype' ,'普通销售' as 'cbustype' ,'发货单' as 'csource' ,GETDATE() as 'ddate' " +
                    "from DispatchList " +
                    "where cDLCode = '"+jHead.getString("cbuscode");
            //查询账套中的发货单信息
            SaleInvEntity U8Obj = new SaleInvEntity();
            U8Obj.setDataSourceCode("YONYOUU8NEW");
            List<HashMap<String, Object>> selDisListMap = saleInvDao.getU8BillData(sqlSelDisList,U8Obj);
            if(CollectionUtils.isEmpty(selDisListMap) ){
                return BaseU8Result.getFailureMessageEntity("发货单主表查询失败"+jHead.getString("cbuscode"));
            }
            HashMap<String, Object> selDis= selDisListMap.get(0);
            if(StrUtil.isEmpty(jHead.getString("dlid"))){
                jHead.put("dlid",selDis.get("dlid"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdlcode"))){
                jHead.put("cdlcode",selDis.get("cdlcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbuscode"))){
                jHead.put("cbuscode",selDis.get("cbuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdepcode"))){
                jHead.put("cdepcode",selDis.get("cdepcode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cpersoncode"))){
                jHead.put("cpersoncode",selDis.get("cpersoncode"));
            }
            if(StrUtil.isEmpty(jHead.getString("ccuscode"))){
                jHead.put("ccuscode",selDis.get("ccuscode"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cinvoicecompany"))){
                jHead.put("cinvoicecompany",selDis.get("cinvoicecompany"));
            }
            if(StrUtil.isEmpty(jHead.getString("cvouchtype"))){
                jHead.put("cvouchtype",selDis.get("cvouchtype"));
            }
            if(StrUtil.isEmpty(jHead.getString("cbustype"))){
                jHead.put("cbustype",selDis.get("cbustype"));
            }
            if(StrUtil.isEmpty(jHead.getString("csource"))){
                jHead.put("csource",selDis.get("csource"));
            }
            if(StrUtil.isEmpty(jHead.getString("ddate"))){
                jHead.put("ddate",selDis.get("ddate"));
            }
            if(StrUtil.isEmpty(jHead.getString("cmemo"))){
                jHead.put("cmemo",selDis.get("cmemo"));
            }
            if(StrUtil.isEmpty(jHead.getString("cdefine1"))){
                jHead.put("cdefine1",selDis.get("cdefine1"));
            }
            //表体
            JSONArray body = new JSONArray();
            JSONArray jBody = jsonObject.getJSONArray("body");
            for (int i = 0; i < jBody.size(); i++) {
                JSONObject jBodyObj = jBody.getJSONObject(i);
                if(StrUtil.isEmpty(jBodyObj.getString("cinvcode"))){
                    resultStr.append("材料编码不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if(StrUtil.isEmpty(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为空 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                if("0".equals(jBodyObj.getString("iquantity"))){
                    resultStr.append("数量不能为0 行："+i+1) ;
                    //如果有错误信息，则返回错误信息
                    if(StrUtil.isNotEmpty(resultStr.toString())){
                        return BaseU8Result.getFailureMessageEntity(resultStr.toString());
                    }
                }
                //发货退货单子表@
                String sqlSelDisLists = " select idlsid  from  "+AccCode+".dbo.DispatchList dl  left join  "+AccCode+".dbo.DispatchLists dls on dls.DLID = dl.DLID  " +
                        "where cDLCode = '"+jBodyObj.getString("cbdlcode")+"" +
                        "' and cInvCode = '"+jBodyObj.getString("cinvcode")+"" +
                        "' and irowno = '"+jBodyObj.getString("cdblrowno")+"'";
                List<HashMap<String, Object>> selDisDetailsListMap = saleInvDao.getU8BillData(sqlSelDisLists,U8Obj);
                if(CollectionUtils.isEmpty(selDisDetailsListMap) ){
                    return BaseU8Result.getFailureMessageEntity("发货单子表查询失败"+jHead.getString("cbuscode")+"发货单表体存货"+jBodyObj.getString("cinvcode")+"行号"+jBodyObj.getString("cdblrowno")+"不存在");
                }
                HashMap<String, Object> selDisDetails= selDisDetailsListMap.get(0);
                if(StrUtil.isEmpty(jBodyObj.getString("idlsid"))){
                    jBodyObj.put("idlsid",selDisDetails.get("idlsid"));
                }


                //获取辅计量单位
                String dtInvSql = "select cInvCode,cInvName,cInvStd,Inv.cComUnitCode,cInvCCode,iChangRate, cPUComUnitCode,cSAComUnitCode,cProductUnit,cSTComUnitCode " +
                        "from "+AccCode+".dbo.Inventory Inv " +
                        "left join "+AccCode+".dbo.ComputationUnit Com On Inv.cComUnitCode = Com.cComUnitCode " +
                        "where cInvCode='" + jBodyObj.getString("cinvcode") +"'";
                List<HashMap<String, Object>> dtInv = saleInvDao.getU8BillData(dtInvSql,U8Obj);
                if(CollectionUtils.isEmpty(dtInv) ){
                    return BaseU8Result.getFailureMessageEntity("ERP没有该物料，请检查，行号："+i+1);
                }
                HashMap<String, Object> dtInvObj= dtInv.get(0);
                //存在辅计量单位则 计算件数 =数量/换算率
                if(StrUtil.isNotEmpty(String.valueOf(dtInvObj.get("cSTComUnitCode")))){
                    jBodyObj.put("cassunit",dtInvObj.get("cSTComUnitCode"));
                    String sqlCassUnit = "select * from "+AccCode+".dbo.ComputationUnit where cComUnitCode='" +dtInvObj.get("cSTComUnitCode")+"'";
                    List<HashMap<String, Object>> dtUnit = saleInvDao.getU8BillData(sqlCassUnit,U8Obj);
                    if(CollectionUtils.isEmpty(dtUnit) ){
                        return BaseU8Result.getFailureMessageEntity("查询单位失败，请检查，行号："+i+1);
                    }
                    HashMap<String, Object> dtUnitObj= dtUnit.get(0);
                    jBodyObj.put("iinvexchrate",dtUnitObj.get("iChangRate"));

                    jBodyObj.put("inum",jBodyObj.getBigDecimal("iquantity").divide(new BigDecimal(String.valueOf(dtUnitObj.get("iChangRate"))),6, RoundingMode.HALF_UP));

                }
                jBodyObj.put("irowno",i+1);
                //默认字段
                //是否需要开票
                if(StrUtil.isEmpty(jBodyObj.getString("bneedbill"))){
                    jBodyObj.put("bneedbill",1);
                }
                //是否存货核算开票
                if(StrUtil.isEmpty(jBodyObj.getString("bIAcreatebill"))){
                    jBodyObj.put("bIAcreatebill",0);
                }
                //是否销售出库开票
                if(StrUtil.isEmpty(jBodyObj.getString("bsaleoutcreatebill"))){
                    jBodyObj.put("bsaleoutcreatebill",0);
                }
                body.add(jBodyObj);
            }
            JSONObject main = new JSONObject();
            main.put("head",jHead);
            main.put("billid",billid);
            main.put("AccId",AccId);
            main.put("head",jHead);
            main.put("body",body);
            main.put("domPosition",new ArrayList<>());
            logger.info("销售出库单新增推送U8："+main.toString());
            String result = U8Util.sengBillToU8(main.toString(), "8000110021");
            logger.info("销售出库单新增推送U8结果："+result);
            JSONObject resultObj = JSON.parseObject(result);
            boolean flag = resultObj.getBoolean("success");
            logger.info(resultObj.toString());
            if(!flag){
                return BaseU8Result.getFailureMessageEntity("新增失败：：："+resultObj.getString("message"));
            }else{
                billCode = resultObj.getString("message");
                //审核
                JSONObject audit = new JSONObject();
                audit.put("billid", UUID.randomUUID());
                audit.put("AccId", AccId);
                audit.put("cVoucherId", billCode);
                logger.info("销售出库单审核："+main.toString());
                String resultAudit = U8Util.sengBillToU8(audit.toString(), "8000110022");
                logger.info("销售出库单审核结果："+result);
                JSONObject resultObjA = JSON.parseObject(resultAudit);
                boolean flagAudit = resultObjA.getBoolean("success");
//                logger.info(resultObj.toString());
                if(!flagAudit){
                    //如果审核失败需要调用删除接口回滚单据
                    JSONObject delete = new JSONObject();
                    delete.put("billid", UUID.randomUUID());
                    delete.put("AccId", AccId);
                    delete.put("cVoucherId", billCode);
                    //删除
                    String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
                    //删除之后直接返回失败信息
                    return BaseU8Result.getFailureMessageEntity("新增正常但审核失败：：："+resultObjA.getString("message"));
                }
            }
            return BaseU8Result.getSuccessMessageEntity(resultObj.getString("message"));
        }  catch (Exception e) {
            //如果审核失败需要调用删除接口回滚单据
            JSONObject delete = new JSONObject();
            delete.put("billid", UUID.randomUUID());
            delete.put("AccId", AccId);
            delete.put("cVoucherId", billCode);
            //删除
            String resultDelete = U8Util.sengBillToU8(delete.toString(), "8000110023");
            logger.info("新增失败："+e.getMessage());
            return BaseU8Result.getFailureMessageEntity("新增失败，请联系U8管理员");
        }
    }


}
